Attribute VB_Name = "FieldDrawing"
Option Explicit

'----------------------------------------------------------------
' Field Drawing

Public Sub ScaleToField(Form As Object)
  Form.Cls
  Form.Width = Form.Width - Form.ScaleWidth + (Game.PlayedLevel.Width * CSng(OBJECT_SIZE * PIXEL))
  Form.Height = Form.Height - Form.ScaleHeight + (Game.PlayedLevel.Height * CSng(OBJECT_SIZE * PIXEL))
End Sub

Public Sub DrawField(Form As Object)
  Dim X As Integer
  Dim Y As Integer
  
  ' Draw the entire field
  Dim Object As StdPicture
  For X = 0 To Game.PlayedLevel.Width - 1
    For Y = 0 To Game.PlayedLevel.Height - 1
      Call DrawFieldItem(Form, X, Y)
    Next
  Next
End Sub

Public Sub DrawFieldItem(Form As Object, Optional ByVal X As Integer = -1, Optional ByVal Y As Integer = -1, Optional ByVal Image As StdPicture)
  ' X,Y is player position if omitted
  If X = -1 Then X = Game.PlayedLevel.Player.X
  If Y = -1 Then Y = Game.PlayedLevel.Player.Y
  
  ' If no image specified, define one
  If Image Is Nothing Then
    If X = Game.PlayedLevel.Player.X And Y = Game.PlayedLevel.Player.Y Then
      ' Image is player
      Set Image = Game.Player(Game.PlayerStyle)
    Else
      ' Image is object in field
      Set Image = Game.Object(Game.PlayedLevel.Fields(X, Y))
    End If
  End If
  
  ' Draw the image
  If Not Image Is Nothing Then Call Form.PaintPicture(Image, X * CLng(OBJECT_SIZE * PIXEL), Y * CLng(OBJECT_SIZE * PIXEL), OBJECT_SIZE * PIXEL, OBJECT_SIZE * PIXEL)
End Sub


'----------------------------------------------------------------
' Show an animation of the player

Public Sub ShowPlayerAnim(Form As Object, Direction As Integer, MoveBox As Boolean)
  Dim I As Integer
  
  InAnimation = True ' Keys should have any effect now!
  
  Select Case Direction
    Case 0
      ' Level complete, JOY JOY!!
      For I = 0 To 3
        Game.PlayerStyle = JOY1
        Call DrawFieldItem(Form)
        Call Sleep(SLEEP_LONGTIME)
        Game.PlayerStyle = JOY2
        Call DrawFieldItem(Form)
        Call Sleep(SLEEP_LONGTIME)
      Next
      
      ' Stand normal
      Game.PlayerStyle = STANDLEFT
      Call DrawFieldItem(Form)
      
      
    Case vbKeyLeft
      ' Make a step to the left
      If Game.PrevStep = vbKeyLeft Then
        Call Sleep(SLEEP_TIME)
        Game.PlayerStyle = WALKLEFT
        Call DrawFieldItem(Form)
      End If
      
      ' Stand left
      Game.PlayerStyle = STANDLEFT
      Call DrawFieldItem(Form)
      Call Sleep(SLEEP_TIME)
      
      ' Push the box if in front of it
      If MoveBox Then
        Game.PlayerStyle = PUSHLEFT
        Call DrawFieldItem(Form)
        Call Sleep(SLEEP_MOVEBOX)
        Game.PlayerStyle = STANDLEFT
      End If
      
      
    Case vbKeyRight
      ' Make a step to the right
      If Game.PrevStep = vbKeyRight Then
        Call Sleep(SLEEP_TIME)
        Game.PlayerStyle = WALKRIGHT
        Call DrawFieldItem(Form)
      End If
      
      ' Stand to the right
      Game.PlayerStyle = STANDRIGHT
      Call DrawFieldItem(Form)
      Call Sleep(SLEEP_TIME)
      
      ' Push the box if in front of it
      If MoveBox Then
        Game.PlayerStyle = PUSHRIGHT
        Call DrawFieldItem(Form)
        Call Sleep(SLEEP_MOVEBOX)
        Game.PlayerStyle = STANDRIGHT
      End If
      
      
  End Select
  
  
  DoEvents ' Key presses during sleep
  InAnimation = False ' Keys may be used now
End Sub


'----------------------------------------------------------------
' Draw Level Complete!!

Sub DrawComplete()
  Dim Text As String
  Text = "Level Complete!"
  frmMain.CurrentX = (frmMain.ScaleWidth - frmMain.TextWidth(Text)) / 2
  frmMain.CurrentY = (frmMain.ScaleHeight - frmMain.TextHeight(Text)) / 2
  frmMain.Print Text
End Sub


'----------------------------------------------------------------
' Get an field item from a offset

Public Function FieldOffset(Direction As Integer, Offset As Integer) As LOCATION
  FieldOffset = Game.PlayedLevel.Player
  
  ' Find out which field is located next to us
  Select Case Direction
    Case vbKeyLeft
      Call Sleep(SLEEP_TIME)
      FieldOffset.X = FieldOffset.X - Offset
      If FieldOffset.X < 0 Then FieldOffset.X = 0
      
    Case vbKeyRight
      Call Sleep(SLEEP_TIME)
      FieldOffset.X = FieldOffset.X + Offset
      If FieldOffset.X > Game.PlayedLevel.Width - 1 Then FieldOffset.X = Game.PlayedLevel.Width - 1
      
    Case vbKeyUp
      Call Sleep(SLEEP_TIME)
      FieldOffset.Y = FieldOffset.Y - Offset
      If FieldOffset.Y < 0 Then FieldOffset.Y = 0
      
    Case vbKeyDown
      Call Sleep(SLEEP_TIME)
      FieldOffset.Y = FieldOffset.Y + Offset
      If FieldOffset.Y > Game.PlayedLevel.Height - 1 Then FieldOffset.Y = Game.PlayedLevel.Height - 1
      
  End Select
End Function
