Attribute VB_Name = "LevelLoader"
Option Explicit


'----------------------------------------------------------------
' Show the common dialog to open the level

Public Function OpenLevelDialog(EditMode As Boolean) As Boolean
  Dim Dialog As New CommonFileDialog
  With Dialog
    .DialogTitle = "Open Level"
    .DefaultDirectory = DATAPATH & "Levels"
    .DefaultExtension = "car"
    .Filter = "Carrier Levels (*.car)|*.car"
    If Not ActiveForm Is Nothing Then .hWndParent = ActiveForm.hWnd
    If .ShowOpen Then
      Call LoadLevel(.FileName, EditMode)
      OpenLevelDialog = True
    End If
  End With
End Function


'----------------------------------------------------------------
' Level loader

Public Sub LoadLevel(FileName As String, EditMode As Boolean)
  Dim FileData As LEVELFILE
  Dim Handle As Integer
  
  ' Get an access handle
  Handle = FreeFile()
  
  ' Copy the entire file into the structure
  Open FileName For Binary Access Read Lock Read Write As #Handle
    Get #Handle, , FileData
  Close #Handle
  
  ' Validate::
  
  ' Is it a carrier level?
  If FileData.ID <> PROGRAM_ID Then
    MsgBox "This is not a valid Carrier level file!", vbCritical, "Carrier: Load Error"
    Exit Sub
  End If
  
  ' Test player position
  If FileData.Contents.Fields(FileData.Contents.PLAYER.X, FileData.Contents.PLAYER.Y) <> SPACE Then
    If EditMode Then
      FileData.Contents.Fields(FileData.Contents.PLAYER.X, FileData.Contents.PLAYER.Y) = 0
    Else
      MsgBox "This is not a valid Carrier level file!" & vbCrLf & "Some data conflicts with other level data in the file.", vbCritical, "Carrier: Load Error"
      Exit Sub
    End If
  End If
  
  ' Test the bounds, just for sure.
  If LBound(FileData.Contents.Fields, 1) <> 0 Or LBound(FileData.Contents.Fields, 2) <> 0 Then
    MsgBox "This is not a valid Carrier level file!" & vbCrLf & "Some data conflicts with other level data in the file.", vbCritical, "Carrier: Load Error"
    Exit Sub
  End If
  If UBound(FileData.Contents.Fields, 1) <> FileData.Contents.Width - 1 Or UBound(FileData.Contents.Fields, 2) <> FileData.Contents.Height - 1 Then
    If EditMode Then
      FileData.Contents.Width = UBound(FileData.Contents.Fields, 1) + 1
      FileData.Contents.Height = UBound(FileData.Contents.Fields, 2) + 1
    Else
      MsgBox "This is not a valid Carrier level file!" & vbCrLf & "Some data conflicts with other level data in the file.", vbCritical, "Carrier: Load Error"
      Exit Sub
    End If
  End If
  
  ' Test if the level is playable
  If Not EditMode Then
    If Not LevelIsPlayable(FileData.Contents) Then
      MsgBox "This level is not playable!", vbExclamation
      Exit Sub
    End If
  End If
  
  ' Load the level into the Game structure
  Game.FileName = FileName
  LSet Game.PlayedLevel = FileData.Contents
  LSet Game.OriginalLevel = FileData.Contents
End Sub


'----------------------------------------------------------------
' Get the next level filename...

Public Function GetNextLevel()
  If Game.FileName <> "" Then
    Dim NewFile As String
    Dim NumLong As Long
    Dim NumID As String
    Dim Pos As Long
    Dim Char As String
    Dim I As Long
    
    ' Should end with ".car"
    If Right(Game.FileName, 4) <> ".car" Then Exit Function
    Pos = Len(Game.FileName) - 4
    
    ' Get the numeric end
    Do
      Char = Mid(Game.FileName, Pos, 1)
      If "0" <= Char And Char <= "9" Then
        Pos = Pos - 1
        NumID = Char & NumID
      Else
        If NumID = "" Then Exit Function
        Exit Do
      End If
    Loop
    
    ' Find out what new number should be used for the level
    NewFile = Left(Game.FileName, Pos)
    NumLong = CLng(NumID) + 1               ' The new number
    Pos = (Len(NumID) - Len(CStr(NumLong))) ' Number of prefix zero's
    NumID = NumLong                         ' Without the prefix zero's
    For I = 1 To Pos
      NumID = "0" & NumID
    Next
    
    ' Test if the file exists...
    NewFile = NewFile & NumID & ".car"
    If FileExist(NewFile) Then
      GetNextLevel = NewFile
      Exit Function
    End If
  End If
End Function



'----------------------------------------------------------------
' Test for level playabilities

Public Function LevelIsPlayable(TestLevel As LEVEL) As Boolean
  Dim X As Integer
  Dim Y As Integer
  Dim TargetCount As Integer
  Dim BoxCount As Integer
  Dim Item As Byte
  
  For X = 0 To TestLevel.Width - 1
    For Y = 0 To TestLevel.Height - 1
      Item = TestLevel.Fields(X, Y)
      If Item = BOX Then
        BoxCount = BoxCount + 1
      ElseIf Item = TARGET Then
        TargetCount = TargetCount + 1
      End If
    Next
  Next
  
  If BoxCount = 0 Or BoxCount <> TargetCount Then
    LevelIsPlayable = False
    Exit Function
  End If
  LevelIsPlayable = True
End Function
